<?php

namespace ManageLicense\Admin;

use Exception;
use ManageLicense\Api\License;
use ManageLicense\Services\addProduct;
use Smarty;
use WHMCS\Database\Capsule as DB;

/**
 * Sample Admin Area Controller
 */
class Controller
{
    public $api;
    public $params;
    public function __construct($params)
    {
        $this->api = new License($params);
        $this->params = $params;
    }
    /**
     * Index action.
     *
     * @param array $vars Module configuration parameters
     *
     * @return string
     */
    public function index($vars)
    {
        $smarty = new Smarty;

        try {
            $allGroupsProduct = $this->getAllGroupProducts();

            $smarty->assign(['allGroupsProduct' => $allGroupsProduct]);
        } catch (\Throwable $th) {
            $smarty->assign(['allGroupsProduct' => null, 'error' => [
                'code' => $th->getCode(),
                'message' => $th->getMessage(),
            ]]);
        }
        $smarty->display(ROOTDIR . '/modules/addons/manage_license/templates/home.tpl');
    }

    /**
     * Show action.
     *
     * @param array $vars Module configuration parameters
     *
     * @return string
     */
    public function show($vars)
    {
        // Get common module parameters
        $modulelink = $vars['modulelink']; // eg. addonmodules.php?module=addonmodule
        $version = $vars['version']; // eg. 1.0
        $LANG = $vars['_lang']; // an array of the currently loaded language variables

        // Get module configuration parameters
        $configTextField = $vars['Text Field Name'];
        $configPasswordField = $vars['Password Field Name'];
        $configCheckboxField = $vars['Checkbox Field Name'];
        $configDropdownField = $vars['Dropdown Field Name'];
        $configRadioField = $vars['Radio Field Name'];
        $configTextareaField = $vars['Textarea Field Name'];
    }

    public function getAllGroupProducts($group = null)
    {
        try {
            $group = !is_null($group) ? '/show' . '/' . $group : $group;

            $res = $this->api->call('get', '/license/group' . $group);

            throw_if($res->getStatusCode() !== 200, new Exception('wrong response, api error code : ' . $res->getStatusCode(), $res->getStatusCode()));

            $arrayRes = json_decode($res->getBody()->getContents(), true);
            foreach ($arrayRes['data'] as $product) {
                $array[$product['code']] = ['name' => $product['name'], 'description' => $product['description']];
            }
            return $array;
        } catch (\Throwable $th) {
            throw new Exception($th->getMessage(), $th->getCode());
        }
    }

    public function getProductsForAjax()
    {
        try {
            $groupName = $_GET['groupName'];

            $path = [
                $groupName => file_exists(ROOTDIR . '/modules/servers/lic_' . strtolower($groupName) . '/lic_' . strtolower($groupName) . '.php') ? true : false,
            ];

            $products = $this->getAllGroupProducts($groupName);

            $smarty = new Smarty;

            $smarty->assign(
                [
                    'path' => $path[$groupName],
                    'products' => $products,
                    'servertype' => 'lic_' . strtolower($groupName),
                    'whmcs_groups' => ($this->getInfo())['whmcs_groups'],
                    'allGroupsProduct' => ($this->getInfo())['allGroupsProduct'],
                    'addedProducts' => ($this->getInfo())['addedProducts'],
                    'addonsProducts' => ($this->getInfo())['addonsProducts'],
                ]
            );

            $view = $smarty->fetch(ROOTDIR . '/modules/addons/manage_license/templates/productTable.tpl');

            ob_clean();
            http_response_code(200);
            echo json_encode(
                [
                    'status' => 'success',
                    'data' => $view,
                ]
            );
            die();
        } catch (\Throwable $th) {
            ob_clean();
            http_response_code(500);
            echo json_encode(
                [
                    'status' => 'error',
                    'message' => $th->getMessage(),
                ]
            );
            die();
        }
    }

    public function addProduct()
    {
        $requirements = [
            'gid' => 'group',
            'productCode' => 'product code',
            'productName' => 'product name',
            'servertype' => 'servertype',
            'description' => 'description',
            'profitPercentage' => 'profit percentage',
            'roundBy' => 'round by',
        ];

        $missed_keys = [];
        foreach ($requirements as $key => $value) {
            if (!isset($_REQUEST[$key]) || is_null($key)) {
                $missed_keys[] = $value;
            }
        }

        if (!empty($missed_keys)) {
            ob_clean();
            http_response_code(500);
            echo json_encode(
                [
                    'status' => 'failed',
                    'message' => 'missing these inputs: ' . json_encode($missed_keys),
                ]
            );
            die();
        }

        $result = (new addProduct($this->params))->execute();

        if ($result['status'] === 'success') {
            ob_clean();
            http_response_code(200);
            echo json_encode(
                [
                    'status' => 'success',
                ]
            );
            die();
        } else {
            ob_clean();
            http_response_code(500);
            echo json_encode(
                [
                    'status' => 'failed',
                    'message' => $result['message'],
                ]
            );
            die();
        }
    }

    public function reInstallModule() //TODO need to completed
    {
        try {
            $moduleName = $_REQUEST['moduleName'];
        } catch (\Throwable $th) {
            ob_clean();
            http_response_code(500);
            echo json_encode(
                [
                    'status' => 'error',
                    'data' => $th->getMessage(),
                ]
            );
            die();
        }
    }

    public function changePage()
    {
        try {
            $smarty = new Smarty;

            $allGroupsProduct = $this->getAllGroupProducts();

            $value = $_REQUEST['value'] !== 'settings' ? $this->getValue($_REQUEST['value']) : null;

            $path = [
                $groupName => file_exists(ROOTDIR . '/modules/servers/lic_' . strtolower($groupName) . '/lic_' . strtolower($groupName) . '.php') ? true : false
            ];

            $smarty->assign([
                'path' => $path[$groupName],
                'allGroupsProduct' => $allGroupsProduct,
                'certumProducts' => $this->getCertumProducts(),
                'value' => $value,
                'whmcs_groups' => $this->getInfo()['whmcs_groups'],
                'allGroupsProduct' => $this->getInfo()['allGroupsProduct'],
                'addedProducts' => $this->getInfo()['addedProducts'],
                'addonsProducts' => $this->getInfo()['addonsProducts'],
            ]);

            $view = $smarty->fetch(ROOTDIR . '/modules/addons/manage_license/templates/' . $_REQUEST['value'] . '.tpl');

            ob_clean();
            http_response_code(200);
            echo json_encode(
                [
                    'status' => 'success',
                    'data' => $view,
                ]
            );
            die();
        } catch (\Throwable $th) {
            ob_clean();
            http_response_code(500);
            echo json_encode(
                [
                    'status' => 'error',
                    'message' => $th->getMessage(),
                ]
            );
            die();
        }
    }

    public function getValue($value)
    {
        $url = $value === 'transactions' ? '/financial/transactions/list' : '/order/license/';

        $res = $this->api->call('get', $url);

        throw_if($res->getStatusCode() !== 200, new Exception('wrong response, api error code : ' . $res->getStatusCode(), $res->getStatusCode()));

        $arrayRes = json_decode($res->getBody()->getContents(), true);

        return $arrayRes['data'];
    }

    public function showGetSchema()
    {
        $smarty = new Smarty;

        try {
            $res = $this->api->call('get', '/order/license/action/' . $_REQUEST['order_id'] . '/getSchema');

            throw_if($res->getStatusCode() !== 200, new Exception('wrong response, api error code : ' . $res->getStatusCode(), $res->getStatusCode()));

            $arrayRes = json_decode($res->getBody()->getContents(), true);

            $smarty->assign([
                'inputs' => $arrayRes["data"]["inputs"],
                'order_id' => $_REQUEST['order_id'],
            ]);

            $smarty->display(ROOTDIR . '/modules/addons/manage_license/templates/getSchema.tpl');
        } catch (\Throwable $th) {
            $smarty->assign([
                'errorMessage' => $th->getMessage(),
            ]);
            $smarty->display(ROOTDIR . '/modules/addons/manage_license/templates/error.tpl');
        }
    }

    public function customAction()
    {
        try {
            $res = $this->api->call('post', '/order/license/action/' . $_REQUEST['order_id'] . '/run', $_REQUEST);

            throw_if($res->getStatusCode() !== 200, new Exception('wrong response, api error code : ' . $res->getStatusCode(), $res->getStatusCode()));

            $arrayRes = json_decode($res->getBody()->getContents(), true);

            ob_clean();
            http_response_code(200);
            echo json_encode(
                [
                    'status' => 'success',
                    'message' => 'عملیات با موفقیت انجام شد',
                ]
            );
            die();
        } catch (\Throwable $th) {
            ob_clean();
            http_response_code(500);
            echo json_encode(
                [
                    'status' => 'error',
                    'message' => $th->getMessage(),
                ]
            );
            die();
        }
    }

    public function getCertumProducts()
    {
        $res = $this->api->call('get', '/ssl/group/show/certum');

        throw_if($res->getStatusCode() !== 200, new Exception('wrong response, api error code : ' . $res->getStatusCode(), $res->getStatusCode()));

        return (json_decode($res->getBody()->getContents(), true))['data'];
    }

    private function getInfo()
    {
        $addonsProducts = DB::table('tbladdons')
            ->join('tblmodule_configuration', 'tbladdons.id', '=', 'tblmodule_configuration.entity_id')
            ->where('tblmodule_configuration.friendly_name', 'select Products')
            ->where('tblmodule_configuration.entity_type', 'addon')
            ->select('tblmodule_configuration.value')
            ->pluck('value');

        $addedProducts = DB::table('tblproducts')->where('servertype', 'LIKE', 'lic_%')->pluck('configoption1')->concat($addonsProducts)->toArray();

        $allGroupsProduct = $this->getAllGroupProducts();

        $whmcs_groups = DB::table('tblproductgroups')->get();

        return [
            'whmcs_groups' => $whmcs_groups,
            'allGroupsProduct' => $allGroupsProduct,
            'addedProducts' => $addedProducts,
            'addonsProducts' => $addonsProducts->toArray(),
        ];
    }

    public function showProductInfo()
    {
        $smarty = new Smarty;

        try {
            $res = $this->api->call('get', '/license/show/' . $_REQUEST['productCode']);

            throw_if($res->getStatusCode() !== 200, new Exception('wrong response, api error code : ' . $res->getStatusCode(), $res->getStatusCode()));

            $arrayRes = json_decode($res->getBody()->getContents(), true);

            $result = $this->api->call('post', '/license/options/' . $_REQUEST['productCode']);

            throw_if($result->getStatusCode() !== 200, new Exception('wrong response, api error code : ' . $result->getStatusCode(), $result->getStatusCode()));

            $arrayResult = json_decode($result->getBody()->getContents(), true);

            $smarty->assign([
                'product' => $arrayRes["data"]["product"],
                'currency' => $arrayRes["data"]["currency"],
                'metadata' => $arrayRes["data"]["metaData"],
                'gOptions' => $arrayResult["data"]['gOptions'],
            ]);

            $smarty->display(ROOTDIR . '/modules/addons/manage_license/templates/showProductInfo.tpl');
        } catch (\Throwable $th) {
            $smarty->assign([
                'errorMessage' => $th->getMessage(),
            ]);
            $smarty->display(ROOTDIR . '/modules/addons/manage_license/templates/error.tpl');
        }
    }

    public function showSSLInfo()
    {
        $smarty = new Smarty;

        try {
            $res = $this->api->call('get', '/ssl/products/show/' . $_REQUEST['productCode']);

            throw_if($res->getStatusCode() !== 200, new Exception('wrong response, api error code : ' . $res->getStatusCode(), $res->getStatusCode()));

            $arrayRes = json_decode($res->getBody()->getContents(), true);

            $smarty->assign([
                'currency' => $arrayRes["data"]["currency"],
                'product' => $arrayRes["data"]["product"],
                'gOptions' => $arrayRes["data"]['group_options'],
            ]);

            $smarty->display(ROOTDIR . '/modules/addons/manage_license/templates/showSSLInfo.tpl');
        } catch (\Throwable $th) {
            $smarty->assign([
                'errorMessage' => $th->getMessage(),
            ]);
            $smarty->display(ROOTDIR . '/modules/addons/manage_license/templates/error.tpl');
        }
    }
}
