<?php

namespace ManageLicense\Api;

use Exception;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\BadResponseException;
use WHMCS\Authentication\CurrentUser;
use WHMCS\Database\Capsule as DB;

class ApiParent
{
    // example: /api/v3
    public $apiVersion;
    // example: resellha.com (without any http:// and https://)
    public $domain;

    public $token;

    public $params;

    public function __construct($params)
    {
        $license_key = DB::table('tbladdonmodules')
            ->where('module', 'manage_license')
            ->where('setting', 'APIKey')
            ->first();
        // get domain and api token from module configs in db
        $characters = 'abcdefghijklmnopqrstuvwxyz';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < 10; $i++) {
            $randomString .= $characters[random_int(0, $charactersLength - 1)];
        }

        $this->apiVersion = '/api/v3';
        $this->domain = $randomString . '.yek.domains';
        $this->params = $params;
        $this->token = $license_key->value;
    }

    /**
     * call api 
     *
     * @param string $method
     * @param string $route have to starts with '/'
     * @param array $data
     * @return object
     */
    public function call(string $method, string $route, array $data = [])
    {
        try {
            //add verify key to denied ssl validation check
            $client = new Client(['base_uri' => 'https://' . $this->domain, 'verify' => false]);

            $http = $client->request($method, $this->apiVersion . $route, [
                // 'debug' => true,
                'json' => $data,
                'headers' => [
                    'Accept'     => 'application/json',
                    'Authorization' => 'Bearer ' . $this->token,
                ]
            ]);

            // throw_if(($http->getStatusCode() != 200), new Exception());
            // logActivity(json_decode($http, true)['message']);
            // echo  '<pre>';var_dump(json_decode($http, true) );die;echo  '</pre>';
            return $http;
        } catch (BadResponseException $e) {
            $currentUser = (new CurrentUser())->client();
            $user = $currentUser ? $currentUser->id : 0;

            $decode = json_decode($e->getResponse()->getBody(), true);

            $error_msg = is_array($decode) && isset($decode['message']) ? $decode['message'] : $e->getResponse()->getReasonPhrase();

            logActivity('manage license (guzzle) : ' . $e->getResponse()->getBody(), $user);

            throw new Exception($error_msg, $e->getResponse()->getStatusCode());
        }
    }

    public function custom_field()
    {
        $customFields = [];
        foreach ($this->params['model']->customFieldValues as $customFieldValue) {
            $customField = $customFieldValue->customField;
            $customFields[$customField->fieldName] = $customFieldValue->value;
        }
        $availableCustomFields = \WHMCS\CustomField::productFields($this->params["pid"])->pluck("fieldname")->all();
        foreach ($availableCustomFields as $fieldName) {
            if (!isset($customFields[$fieldName])) {
                $customFields[$fieldName] = "";
            }
        }
        foreach ($customFields as $fieldName => $fieldValue) {
            unset($customFields[$fieldName]);
            if (strpos($fieldName, "|")) {
                $parts = explode("|", $fieldName);
                $fieldName = trim($parts[0]);
            }
            if (strpos($fieldValue, "|")) {
                $parts = explode("|", $fieldValue);
                $fieldValue = trim($parts[0]);
            }
            $customFields[$fieldName] = \WHMCS\Input\Sanitize::encode(strip_tags(\WHMCS\Input\Sanitize::decode($fieldValue)));
        }
        unset($customFields['orderid']);

        $this->params["customfields"] = $customFields;

        return $this->params["customfields"];
    }

    // public function options()
    // {
    //     $configOptions = [];
    //     $configOptionsData = \WHMCS\Database\Capsule::table("tblproductconfigoptions")->join("tblhostingconfigoptions", "tblproductconfigoptions.id", "=", "tblhostingconfigoptions.configid")->join("tblproductconfigoptionssub", "tblproductconfigoptionssub.id", "=", "tblhostingconfigoptions.optionid")->join("tblproductconfiglinks", "tblproductconfiglinks.gid", "=", "tblproductconfigoptions.gid")
    //         ->where("tblhostingconfigoptions.relid", "=", $this->params['accountid'])->where("tblproductconfiglinks.pid", "=", $this->params['packageid'])->get(["tblproductconfigoptions.optionname as productConfigOptionName", "tblproductconfigoptions.optiontype", "tblproductconfigoptionssub.optionname", "tblhostingconfigoptions.qty"])->all();
    //     foreach ($configOptionsData as $configOptionData) {
    //         $configOptionName = $configOptionData->productConfigOptionName;
    //         $configOptionType = (int) $configOptionData->optiontype;
    //         $configOptionValue = $configOptionData->optionname;
    //         $configOptionQuantity = $configOptionData->qty;
    //         if (strpos($configOptionName, "|")) {
    //             $configOptionName = explode("|", $configOptionName);
    //             $configOptionName = trim($configOptionName[0]);
    //         }
    //         if (strpos($configOptionValue, "|")) {
    //             $configOptionValue = explode("|", $configOptionValue);
    //             $configOptionValue = trim($configOptionValue[0]);
    //         }
    //         if (in_array($configOptionType, [3, 4])) {
    //             $configOptionValue = $configOptionQuantity;
    //         }
    //         $configOptions[$configOptionName] = $configOptionValue;
    //     }
    //     echo '<pre>';
    //     var_dump($configOptions);
    //     die;
    //     $params["configoptions"] = $configOptions;
    // }
}
