<?php

namespace ManageLicense\Services;

use Exception;
use ManageLicense\Api\License;
use WHMCS\Database\Capsule as DB;

class addProduct
{
    public $params;
    public function __construct($params)
    {
        $this->params = $params;
    }
    public function execute()
    {
        try {
            $productType = $_REQUEST['isAddon'] === 'on' ? 'addon' : 'product';

            DB::beginTransaction();

            $product = $this->addProduct($productType);

            $this->addCustomFields($product, $productType);

            if ($_REQUEST['isAddon'] != 'on') {
                $configGroup = $this->addConfigGroup($product, $productType);

                $this->addConfigLink($product, $configGroup);

                $this->addConfigOption($configGroup);
            }
            DB::commit();

            return ['status' => 'success'];
        } catch (\Throwable $th) {
            DB::rollBack();
            logActivity($th);

            return ['status' => 'error', 'message' => $th->getMessage()];
        }
    }

    private function addProduct($productType)
    {
        if ($_REQUEST['isAddon'] === 'on') {

            $addonId = DB::table('tbladdons')->insertGetId([
                'name' => $_REQUEST['productName'], 'billingcycle' => 'recurring', 'module' => $_REQUEST['servertype'], 'description' => $_REQUEST['description'],
            ]);

            DB::table('tblmodule_configuration')->insert([
                'entity_type' => 'addon', 'entity_id' => $addonId, 'setting_name' => 'configoption1', 'friendly_name' => 'select Products', 'value' => $_REQUEST['productCode'],
            ]);

            $product = DB::table('tbladdons')->where('id', $addonId)->first();
        } else {
            $tbl = DB::table('tblproducts')->where('servertype', $_REQUEST['servertype'])->where('configoption1', $_REQUEST['productCode']);
            if ($tbl->exists()) {
                $tbl->update([
                    'type' => 'product',
                    'gid' => $_REQUEST['gid'],
                    'name' => $_REQUEST['productName'],
                    'paytype' => 'recurring',
                    'autosetup' => 'payment',
                ]);
            } else {
                DB::table('tblproducts')->insert([
                    'type' => 'product',
                    'servertype' => $_REQUEST['servertype'],
                    'configoption1' => $_REQUEST['productCode'],
                    'gid' => $_REQUEST['gid'],
                    'name' => $_REQUEST['productName'],
                    'description' => $_REQUEST['description'],
                    'paytype' => 'recurring',
                    'autosetup' => 'payment',
                ]);
            }

            $product = DB::table('tblproducts')->where('servertype', $_REQUEST['servertype'])->where('configoption1', $_REQUEST['productCode'])->first();
        }

        return $product;
    }

    private function addCustomFields($product, $productType)
    {
        if ($_REQUEST['productGroup'] == 'ssl') {
            $metadata = [];
        } else {
            $res = (new License($this->params))->call('get', '/license/show/' . $_REQUEST['productCode']);

            throw_if($res->getStatusCode() !== 200, new Exception('wrong response, api error code : ' . $res->getStatusCode(), $res->getStatusCode()));

            $arrayRes = json_decode($res->getBody()->getContents(), true);

            $metadata = $arrayRes['data']['metaData'];
        }

        $this->addPrice($product->id, $arrayRes['data']['product']['prices'], $productType);

        foreach ($metadata as $item) {
            DB::table('tblcustomfields')->updateOrInsert(
                [
                    'relid' => $product->id, 'type' => $productType, 'fieldname' => $item['id'] . '|' . $item['name'],
                ],
                [
                    'fieldtype' => $item['type'], 'required' => $item['type'] ? 'on' : '',
                ]
            );
        }

        DB::table('tblcustomfields')->updateOrInsert(
            [
                'relid' => $product->id, 'type' => $productType, 'fieldname' => 'orderid',
            ],
            [
                'fieldtype' => 'text', 'adminonly' => 'on',
            ]
        );
    }

    private function addConfigGroup($product, $productType)
    {
        DB::table('tblproductconfiggroups')->updateOrInsert(
            [
                'name' => $product->name . ' options',
            ],
            [
                'description' => 'for ' . $productType . ' ' . $product->id,
            ]
        );

        $configGroup = DB::table('tblproductconfiggroups')->where('name', $product->name . ' options')->first();

        return $configGroup;
    }

    private function addConfigLink($product, $configGroup)
    {
        DB::table('tblproductconfiglinks')->updateOrInsert(
            [
                'pid' => $product->id,
            ],
            [
                'gid' => $configGroup->id,
            ]
        );
    }

    private function addConfigOption($configGroup)
    {
        if ($_REQUEST['productGroup'] == 'ssl') {
            $res = (new License($this->params))->call('get', '/ssl/products/show/' . $_REQUEST['productCode']);

            throw_if($res->getStatusCode() !== 200, new Exception('wrong response, api error code : ' . $res->getStatusCode(), $res->getStatusCode()));

            $arrayRes = json_decode($res->getBody()->getContents(), true);

            $gOptions = $arrayRes['data']['group_options'];

            $prices = $arrayRes['data']['group_options']['options']['prices'];
        } else {
            $res = (new License($this->params))->call('post', '/license/options/' . $_REQUEST['productCode']);

            throw_if($res->getStatusCode() !== 200, new Exception('wrong response, api error code : ' . $res->getStatusCode(), $res->getStatusCode()));

            $arrayRes = json_decode($res->getBody()->getContents(), true);

            $gOptions = $arrayRes['data']['gOptions'];

            $prices = $arrayRes['data']['gOptions']['options']['prices'];
        }

        foreach ($gOptions as $group) {
            $optiontype = $group['type'] === 'select' ? 1 : 4;
            DB::table('tblproductconfigoptions')->updateOrInsert(
                [
                    'optionname' => $group['id'] . '|' . $group['title'],
                    'gid' => $configGroup->id,
                ],
                [
                    'optiontype' => $optiontype,
                    'qtyminimum' => array_key_exists('min', $group) ? $group['min'] : 0,
                    'qtymaximum' => array_key_exists('max', $group) ? $group['max'] : 0,
                ]
            );
            $configOptions = DB::table('tblproductconfigoptions')->where('gid', $configGroup->id)->where('optionname', $group['id'] . '|' . $group['title'])->first();
            if ($optiontype == 1) {
                $optionSub = DB::table('tblproductconfigoptionssub')->insertGetId(
                    [
                        'optionname' => 'None|None',
                        'configid' => $configOptions->id,
                    ]
                );
                $array = [
                    'monthly' => ['price' => 0, 'setup' => 0], 'quarterly' => ['price' => 0, 'setup' => 0], 'semiannually' => ['price' => 0, 'setup' => 0], 'annually' => ['price' => 0, 'setup' => 0], 'biennially' => ['price' => 0, 'setup' => 0], 'triennially' => ['price' => 0, 'setup' => 0],
                ];
                $this->addPrice($optionSub, $array, 'configoptions');
            }
            foreach ($group['options'] as $option) {
                $optionSub = DB::table('tblproductconfigoptionssub')->where('configid', $configOptions->id)->where('optionname', $option['id'] . '|' . $option['title'])->first();

                if (is_null($optionSub)) {
                    $optionSub = DB::table('tblproductconfigoptionssub')->insertGetId(
                        [
                            'optionname' => $option['id'] . '|' . $option['title'],
                            'configid' => $configOptions->id,
                        ]
                    );
                } else {
                    $optionSub = $optionSub->id;
                }

                $this->addPrice($optionSub, $option['price'], 'configoptions');
            }
        }
    }

    private function addPrice($productId, $prices, $type = 'product')
    {
        $array = [
            'msetupfee' => 0.00, 'qsetupfee' => 0.00, 'ssetupfee' => 0.00, 'asetupfee' => 0.00, 'bsetupfee' => 0.00, 'tsetupfee' => 0.00,
            'monthly' => -1.00, 'quarterly' => -1.00, 'semiannually' => -1.00, 'annually' => -1.00, 'biennially' => -1.00, 'triennially' => -1.00,
            'currency' => 1,
        ];

        foreach ($prices as $cycle => $price) {
            foreach ($array as $key => $value) {
                if ($cycle === $key) {
                    $array[$key] = $price['price'] + ($price['price'] * $_REQUEST['profitPercentage'] / 100);
                    $array[str_split($key)[0] . 'setupfee'] = $price['setup'] + ($price['setup'] * $_REQUEST['profitPercentage'] / 100);
                    if ($_REQUEST['roundBy'] < 10) {
                        $array[$key] = round($array[$key], $_REQUEST['roundBy']);
                        $array[str_split($key)[0] . 'setupfee'] = round($array[str_split($key)[0] . 'setupfee'], $_REQUEST['roundBy']);
                    }
                }
            }
        }

        $aa = DB::table('tblpricing')->updateOrInsert(
            [
                'relid' => $productId, 'type' => $type,
            ],
            $array
        );
    }
}
